/**
 * @default ngx-scanner-face
 * @author <DaiDH>
 * @package https://www.npmjs.com/package/ngx-scanner-face
 * @license MIT https://github.com/id1945/ngx-scanner-face/blob/master/LICENSE
 * @copyright <https://github.com/id1945>
*/
import { ElementRef, EventEmitter, OnDestroy, OnInit } from '@angular/core';
import { AsyncSubject, BehaviorSubject } from 'rxjs';
import { Config, Device, DrawOptions, Env, Result, ScannerFaceConfig } from './ngx-scanner-face.options';
import * as i0 from "@angular/core";
export declare class NgxScannerFaceComponent implements OnInit, OnDestroy {
    /**
     * Element
     * playsinline required to tell iOS safari we don't want fullscreen
     */
    video: ElementRef<HTMLVideoElement>;
    canvas: ElementRef<HTMLCanvasElement>;
    /**
     * EventEmitter
     */
    event: EventEmitter<Result>;
    /**
     * Input
     */
    src: string;
    isAuto: boolean;
    fps: number;
    timeoutDetect: number;
    constraints: MediaStreamConstraints | any;
    env: Partial<Env>;
    draw: Partial<DrawOptions>;
    human: Partial<Config>;
    config: ScannerFaceConfig;
    style: any;
    videoStyle: any;
    canvasStyle: any;
    /**
     * Export
    */
    isStart: boolean;
    isPause: boolean;
    isLoading: boolean;
    isTorch: boolean;
    data: BehaviorSubject<Partial<Result>>;
    devices: BehaviorSubject<Device[]>;
    deviceIndexActive: number;
    private H_OBJ;
    private detectionLoop;
    private drawLoop;
    private interpolated;
    private STATUS;
    ngOnInit(): void;
    /**
     * start
     * @param playDeviceCustom
     * @returns
     */
    start(playDeviceCustom?: Function): AsyncSubject<any>;
    /**
     * playDevice
     * @param deviceId
     * @param as
     * @returns
     */
    playDevice(deviceId: string, as?: AsyncSubject<any>): AsyncSubject<any>;
    /**
     * stop
     * @return AsyncSubject
     */
    stop(): AsyncSubject<any>;
    /**
     * play
     * @return AsyncSubject
     */
    play(): AsyncSubject<any>;
    /**
     * pause
     * @return AsyncSubject
     */
    pause(): AsyncSubject<any>;
    /**
     * torcher
     * @returns
     */
    torcher(): AsyncSubject<any>;
    /**
     * loadImage
     * @param src
     */
    loadImage(src: string): AsyncSubject<any>;
    /**
     * applyConstraints
     * @param constraints
     * @param deviceIndex
     * @returns
     */
    applyConstraints(constraints: MediaTrackConstraintSet | MediaTrackConstraints | any, deviceIndex?: number): AsyncSubject<any>;
    /**
     * getConstraints
     * @param deviceIndex
     * @returns
     */
    getConstraints(deviceIndex?: number): MediaTrackConstraintSet | MediaTrackConstraints;
    /**
     * download
     * @param fileName
     * @return AsyncSubject
     */
    download(fileName?: string): AsyncSubject<Result>;
    /**
     * Override Config
     */
    private overrideConfig;
    /**
     * Override Env
     * @return void
     */
    private overrideEnv;
    /**
     * Override drawOptions
     * @return void
     */
    private overrideDraw;
    /**
     * Fix issue on safari
     * https://webrtchacks.com/guide-to-safari-webrtc
     * @param as
     * @param playDeviceCustom
     */
    private safariWebRTC;
    /**
     * loadAllDevices
     * @param as
     * @param playDeviceCustom
     */
    private loadAllDevices;
    /**
     * eventEmit
     * @param response
     */
    private eventEmit;
    ngOnDestroy(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgxScannerFaceComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgxScannerFaceComponent, "ngx-scanner-face", ["scanner"], { "src": { "alias": "src"; "required": false; }; "isAuto": { "alias": "isAuto"; "required": false; }; "fps": { "alias": "fps"; "required": false; }; "timeoutDetect": { "alias": "timeoutDetect"; "required": false; }; "constraints": { "alias": "constraints"; "required": false; }; "env": { "alias": "env"; "required": false; }; "draw": { "alias": "draw"; "required": false; }; "human": { "alias": "human"; "required": false; }; "config": { "alias": "config"; "required": false; }; "style": { "alias": "style"; "required": false; }; "videoStyle": { "alias": "videoStyle"; "required": false; }; "canvasStyle": { "alias": "canvasStyle"; "required": false; }; }, { "event": "event"; }, never, ["*"], false, never>;
}
